package sun.font;

import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import sun.java2d.Disposer;
import java.lang.ref.WeakReference;
import java.lang.reflect.Constructor;
import sun.java2d.DisposerRecord;

public abstract class FontScaler implements DisposerRecord
{
    private static FontScaler nullScaler;
    private static Constructor<FontScaler> scalerConstructor;
    protected WeakReference<Font2D> font;
    protected long nativeScaler;
    protected boolean disposed;
    
    public FontScaler() {
        this.font = null;
        this.nativeScaler = 0L;
        this.disposed = false;
    }
    
    public static FontScaler getScaler(Font2D obj, int i, boolean b, int j) {
        FontScaler nullScaler;
        try {
            nullScaler = (FontScaler)FontScaler.scalerConstructor.newInstance(new Object[] { obj, Integer.valueOf(i), Boolean.valueOf(b), Integer.valueOf(j) });
            Disposer.addObjectRecord(obj, nullScaler);
        }
        catch (Throwable t) {
            nullScaler = FontScaler.nullScaler;
            FontManagerFactory.getInstance().deRegisterBadFont(obj);
        }
        return nullScaler;
    }
    
    public static synchronized FontScaler getNullScaler() {
        if (FontScaler.nullScaler == null) {
            FontScaler.nullScaler = new NullFontScaler();
        }
        return FontScaler.nullScaler;
    }
    
    abstract StrikeMetrics getFontMetrics(long p0) throws FontScalerException;
    
    abstract float getGlyphAdvance(long p0, int p1) throws FontScalerException;
    
    abstract void getGlyphMetrics(long p0, int p1, Point2D.Float p2) throws FontScalerException;
    
    abstract long getGlyphImage(long p0, int p1) throws FontScalerException;
    
    abstract Rectangle2D.Float getGlyphOutlineBounds(long p0, int p1) throws FontScalerException;
    
    abstract GeneralPath getGlyphOutline(long p0, int p1, float p2, float p3) throws FontScalerException;
    
    abstract GeneralPath getGlyphVectorOutline(long p0, int[] p1, int p2, float p3, float p4) throws FontScalerException;
    
    @Override
    public void dispose() {
    }
    
    abstract int getNumGlyphs() throws FontScalerException;
    
    abstract int getMissingGlyphCode() throws FontScalerException;
    
    abstract int getGlyphCode(char p0) throws FontScalerException;
    
    abstract long getLayoutTableCache() throws FontScalerException;
    
    abstract Point2D.Float getGlyphPoint(long p0, int p1, int p2) throws FontScalerException;
    
    abstract long getUnitsPerEm();
    
    abstract long createScalerContext(double[] p0, int p1, int p2, float p3, float p4, boolean p5);
    
    abstract void invalidateScalerContext(long p0);
    
    static {
        FontScaler.nullScaler = null;
        FontScaler.scalerConstructor = null;
        Class[] parameterTypes = { Font2D.class, Integer.TYPE, Boolean.TYPE, Integer.TYPE };
        Class<Object> clazz;
        try {
            if (FontUtilities.isOpenJDK) {
                clazz = Class.forName("sun.font.FreetypeFontScaler");
            }
            else {
                clazz = Class.forName("sun.font.T2KFontScaler");
            }
        }
        catch (ClassNotFoundException ex) {
            clazz = NullFontScaler.class;
        }
        try {
            FontScaler.scalerConstructor = clazz.getConstructor(parameterTypes);
        }
        catch (NoSuchMethodException ex2) {}
    }
}
